<?php

require_once(dirname(__FILE__).'/config.php');
require_once(dirname(__FILE__).'/migrator.php');
require_once(dirname(__FILE__).'/postmeta.php');
require_once(dirname(__FILE__).'/php.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Option class handles the loading and storing of the
// plugin option including all needed conversion routines
// ----------------------------------------------------------------------------

class DiscloseSecret_Option
{
	var $m_allowed_wp_versions = array('2.0.7');
	var $m_cooperation_status = DiscloseSecret_STATUS_UNINITIALIZED;
	var $m_version = DiscloseSecret_VERSION;
	var $m_caps = array();
	var $m_roles = array();
	var $m_users = array();
	var $m_feed_options = array();
	var $m_feed_default_option;
	var $m_users_initialized = false;
	var $m_caps_and_roles_initialized = false;

	// default option values will be set during load()
	var $m_last_wp_version;
	var $m_override_plugin_deactivation;
	var $m_feed_privacy;
	var $m_feed_teaser;
	var $m_rewrite_feed_link;
	var $m_print_cap_form;
	var $m_print_role_form;
	var $m_print_user_form;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Option()
	{
		$this->init_constants();
		$this->init();
	}

	function init_users()
	{
		if ($this->m_users_initialized)
			return;
		$this->m_users_initialized = true;

		global $wpdb;
		$query = "SELECT `ID` AS user_id, `display_name` FROM `{$wpdb->users}` ORDER BY `display_name`";
		$wpdb->query($query);
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();
		foreach ($s as $i)
		{
			$this->m_users[$i->user_id] = array(
				DiscloseSecret_INDEX_ID                => $i->user_id,
				DiscloseSecret_INDEX_NAME              => $i->display_name,
				DiscloseSecret_INDEX_CAN_SEE_PROTECTED => false, /* late init with set_can_see_X */
				DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false  /* late init with set_can_see_X */
			);
		}
	}

	function init_caps_and_roles()
	{
		if ($this->m_caps_and_roles_initialized)
			return;
		$this->m_caps_and_roles_initialized = true;

		// paranoidly force init of role system again by starting a dummy request
		global $wp_roles;
		get_role('Administrator');
		foreach ($wp_roles->role_objects as $role)
		{
			$this->m_roles[$role->name] = array(
				DiscloseSecret_INDEX_ID                => $role->name,
				DiscloseSecret_INDEX_NAME              => ucwords(str_replace('-', ' ', str_replace('_', ' ', $role->name))),
				DiscloseSecret_INDEX_CAN_SEE_PROTECTED => false,
				DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false  /* late init with set_can_see_X */
			);
			foreach ($role->capabilities as $cap => $grant)
			{
				$this->m_caps[$cap] = array(
					DiscloseSecret_INDEX_ID                => $cap,
					DiscloseSecret_INDEX_NAME              => ucwords(str_replace('-', ' ',str_replace('_', ' ', $cap))),
					DiscloseSecret_INDEX_CAN_SEE_PROTECTED => ($cap == 'edit_others_posts'),
					DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false  /* always false */
				);
				if ($this->m_caps[$cap][DiscloseSecret_INDEX_CAN_SEE_PROTECTED])
					$this->m_roles[$role->name][DiscloseSecret_INDEX_CAN_SEE_PROTECTED] = true;
			}
		}
		ksort($this->m_caps);
		ksort($this->m_roles);
	}

	// initializes plugin data
	function init_constants()
	{
		// init constants
		$this->m_feed_options = array(
			DiscloseSecret_OPTION_FULL_CONTENT => __('Full Content', DiscloseSecret_PLUGIN_ID),
			DiscloseSecret_OPTION_EXCERPT_ONLY => __('Excerpt Only', DiscloseSecret_PLUGIN_ID),
			DiscloseSecret_OPTION_TITLE_ONLY   => __('Title Only', DiscloseSecret_PLUGIN_ID)
		);
		$this->m_feed_default_option = __('Default', DiscloseSecret_PLUGIN_ID);
	}

	// Loads plugin option and probably upgrade older versions of the plugin
	function init()
	{
		$version = $this->detect_plugin_version();
		$status = $this->upgrade($version);
		$this->load();
		$this->set_cooperation_status($status);
	}

	// ----------------------------------------------------------------------------
	// option handling
	// ----------------------------------------------------------------------------

	// Upgrades plugin from previous versions or even installs it
	function upgrade($old_version)
	{
		while ($old_version != DiscloseSecret_VERSION)
		{
			$this->load();
			if ($old_version == '0')
			{
				// this is first installation of the plugin
				$old_version = '1.2';
			}
			else if ($old_version == '1.2')
			{
				// migrate predecessor restriction settings
				$old_version = '2.0';
				$migrator =& new DiscloseSecret_Migrator();
				$migrator->migrate_post_restrictions(true);
				// just reload options after plugin name change
				$this->load();
			}
			else if ($old_version == '2.0')
			{
				// because of issues with the wp-plugins.net database, version 2.0.1 and 2.1 are synonymns
				$old_version = '2.0.1';
			}
			else if ($old_version == '2.0.1')
			{
				$old_version = '2.2';
			}
			else if ($old_version == '2.2')
			{
				$old_version = '2.3';
			}
			else
			{
				$this->m_version = $old_version;
				return DiscloseSecret_STATUS_PLUGIN_VERSION_MISMATCH;
			}
			$this->m_version = $old_version;
			// deactivate the overrider each time the plugin was upgraded
			$this->m_override_plugin_deactivation = false;
			$this->save();
		}
		return DiscloseSecret_STATUS_OKAY;
	}

	function save()
	{
		// introduced in 2.0 but we save it anyway during migration
		$option[DiscloseSecret_OPTION_VERSION] = $this->m_version;

		// always in
		$option[DiscloseSecret_OPTION_FEED_PRIVACY] = $this->m_feed_privacy;

		// always in
		$option[DiscloseSecret_OPTION_REWRITE_FEED_LINK] = $this->m_rewrite_feed_link;

		// introduced in 2.0
		if (!(substr($this->m_version, 0, 1) == '1'))
			$option[DiscloseSecret_OPTION_FEED_TEASER] = $this->m_feed_teaser;

		// introduced in 2.2
		if (!(substr($this->m_version, 0, 1) == '1' || substr($this->m_version, 0, 3) == '2.0'))
		{
			$option[DiscloseSecret_OPTION_LAST_WP_VERSION] = $this->m_last_wp_version;
			$option[DiscloseSecret_OPTION_OVERRIDE_PLUGIN_DEACTIVATION] = $this->m_override_plugin_deactivation;
		}

		// introduced in 2.3
		if (!(substr($this->m_version, 0, 1) == '1' || substr($this->m_version, 0, 3) == '2.0'
			 || substr($this->m_version, 0, 3) == '2.2'))
		{
			$option[DiscloseSecret_OPTION_PRINT_CAP_FORM] = $this->m_print_cap_form;
			$option[DiscloseSecret_OPTION_PRINT_ROLE_FORM] = $this->m_print_role_form;
			$option[DiscloseSecret_OPTION_PRINT_USER_FORM] = $this->m_print_user_form;
		}

		update_option(DiscloseSecret_PLUGIN_ID, $option);
	}

	function load()
	{
		global $wp_version;

		$option = get_option(DiscloseSecret_PLUGIN_ID);

		// always in
		if (isset($option[DiscloseSecret_OPTION_FEED_PRIVACY]))
			$this->m_feed_privacy = $option[DiscloseSecret_OPTION_FEED_PRIVACY];
		else
			$this->m_feed_privacy = DiscloseSecret_OPTION_TITLE_ONLY;

		// always in
		if (isset($option[DiscloseSecret_OPTION_REWRITE_FEED_LINK]))
			$this->m_rewrite_feed_link = $option[DiscloseSecret_OPTION_REWRITE_FEED_LINK];
		else
			$this->m_rewrite_feed_link = true;

		// introduced in 2.0
		if (isset($option[DiscloseSecret_OPTION_FEED_TEASER]))
			$this->m_feed_teaser = $option[DiscloseSecret_OPTION_FEED_TEASER];
		else
			$this->m_feed_teaser = __("View the content of this <a href='%permalink%'>protected post</a>", DiscloseSecret_PLUGIN_ID);

		// introduced in 2.2
		if (isset($option[DiscloseSecret_OPTION_LAST_WP_VERSION]))
			$this->m_last_wp_version = $option[DiscloseSecret_OPTION_LAST_WP_VERSION];
		else
			$this->m_last_wp_version = $wp_version;

		// introduced in 2.2
		// deactivate the overrider each time the wordpress was upgraded
		if ($this->m_last_wp_version != $wp_version)
		{
			$this->m_override_plugin_deactivation = false;
			$this->m_last_wp_version = $wp_version;
			$this->save();
		}
		else if (isset($option[DiscloseSecret_OPTION_OVERRIDE_PLUGIN_DEACTIVATION]))
			$this->m_override_plugin_deactivation = $option[DiscloseSecret_OPTION_OVERRIDE_PLUGIN_DEACTIVATION];
		else
			$this->m_override_plugin_deactivation = false;

		// introduced in 2.3
		if (isset($option[DiscloseSecret_OPTION_PRINT_CAP_FORM]))
			$this->m_print_cap_form = $option[DiscloseSecret_OPTION_PRINT_CAP_FORM];
		else
			$this->m_print_cap_form = true;

		// introduced in 2.3
		if (isset($option[DiscloseSecret_OPTION_PRINT_ROLE_FORM]))
			$this->m_print_role_form = $option[DiscloseSecret_OPTION_PRINT_ROLE_FORM];
		else
			$this->m_print_role_form = true;

		// introduced in 2.3
		if (isset($option[DiscloseSecret_OPTION_PRINT_USER_FORM]))
			$this->m_print_user_form = $option[DiscloseSecret_OPTION_PRINT_USER_FORM];
		else
			$this->m_print_user_form = true;
	}

	// ----------------------------------------------------------------------------
	// tools
	// ----------------------------------------------------------------------------

	function is_wp_version_compatible()
	{
		global $wp_version;

		return in_array($wp_version, $this->m_allowed_wp_versions);
	}

	function detect_plugin_version()
	{
		$option = get_option(DiscloseSecret_PLUGIN_ID);
		if ($option === false)
		{
			$option = get_option(DiscloseSecret_POSTRESTRICTIONS_PLUGIN_ID);
			if ($option === false)
				$version = '0';
			else
				$version = '1.2';
		}
		else
			$version = $option[DiscloseSecret_OPTION_VERSION];
		return $version;
	}

	function set_cooperation_status($previous_error = DiscloseSecret_STATUS_OKAY)
	{
		if ($this->m_cooperation_status == DiscloseSecret_STATUS_PLUGIN_VERSION_MISMATCH)
			return;

		if ($previous_error != DiscloseSecret_STATUS_OKAY)
			$this->m_cooperation_status = $previous_error;
		else if (!$this->is_wp_version_compatible() && !$this->m_override_plugin_deactivation)
			$this->m_cooperation_status = DiscloseSecret_STATUS_WP_VERSION_MISMATCH;
		else
			$this->m_cooperation_status = DiscloseSecret_STATUS_OKAY;
	}

	// ----------------------------------------------------------------------------
	// access
	// ----------------------------------------------------------------------------

	function set_from_POST()
	{
		$this->m_override_plugin_deactivation
			= isset($_POST[DiscloseSecret_POST_OVERRIDE_PLUGIN_DEACTIVATION]);
		$this->m_feed_privacy = isset($this->m_feed_options[$_POST[DiscloseSecret_POST_FEED_PRIVACY]])
			? $_POST[DiscloseSecret_POST_FEED_PRIVACY] : DiscloseSecret_OPTION_TITLE_ONLY;
		$this->m_feed_teaser
			= stripslashes(htmlspecialchars_decode($_POST[DiscloseSecret_POST_FEED_TEASER], ENT_QUOTES));
		$this->m_rewrite_feed_link
			= isset($_POST[DiscloseSecret_POST_REWRITE_FEED_LINK]);
		$this->m_print_cap_form
			= isset($_POST[DiscloseSecret_POST_PRINT_CAP_FORM]);
		$this->m_print_role_form
			= isset($_POST[DiscloseSecret_POST_PRINT_ROLE_FORM]);
		$this->m_print_user_form
			= isset($_POST[DiscloseSecret_POST_PRINT_USER_FORM]);
		$this->set_cooperation_status();
	}

	function set_can_see_X($postmeta)
	{
		global $wp_roles;

		if ($this->m_print_role_form)
		{
			$roles =& $this->get_roles();
			foreach (array_keys($roles) as $key)
			{
				$implicit = false;
				$r = $wp_roles->get_role($roles[$key][DiscloseSecret_INDEX_ID]);
				foreach ($postmeta->get_caps() as $c)
				{
					if ($r->has_cap($c))
					{
						$implicit = true;
						break;
					}
				}
				$roles[$key][DiscloseSecret_INDEX_CAN_SEE_IMPLICIT] = $implicit;
			}
		}

		if ($this->m_print_user_form)
		{
			$users =& $this->get_users();
			foreach (array_keys($users) as $key)
			{
				$implicit = false;
				$u =& new WP_User($users[$key][DiscloseSecret_INDEX_ID]);
				foreach ($postmeta->get_caps() as $c)
				{
					if ($u->has_cap($c))
					{
						$implicit = true;
						break;
					}
				}
				if (!$implicit)
				{
					foreach ($postmeta->get_roles() as $r)
					{
						if ($u->has_cap($r))
						{
							$implicit = true;
							break;
						}
					}
				}
				$users[$key][DiscloseSecret_INDEX_CAN_SEE_PROTECTED] = $u->has_cap('edit_others_posts');
				$users[$key][DiscloseSecret_INDEX_CAN_SEE_IMPLICIT] = $implicit;
			}
		}
	}

	function &get_caps()
	{
		$this->init_caps_and_roles();
		return $this->m_caps;
	}

	function &get_roles()
	{
		$this->init_caps_and_roles();
		return $this->m_roles;
	}

	function &get_users()
	{
		$this->init_users();
		return $this->m_users;
	}

	function get_allowed_wp_versions()
	{
		return $this->m_allowed_wp_versions;
	}

	function get_cooperation_status()
	{
		return $this->m_cooperation_status;
	}

	function get_version()
	{
		return $this->m_version;
	}

	function get_feed_options()
	{
		return $this->m_feed_options;
	}

	function get_feed_default_option()
	{
		return $this->m_feed_default_option;
	}

	function get_feed_privacy()
	{
		return $this->m_feed_privacy;
	}

	function get_feed_teaser()
	{
		return $this->m_feed_teaser;
	}

	function get_rewrite_feed_link()
	{
		return $this->m_rewrite_feed_link;
	}

	function get_override_plugin_deactivation()
	{
		return $this->m_override_plugin_deactivation;
	}

	function get_print_cap_form()
	{
		return $this->m_print_cap_form;
	}

	function get_print_role_form()
	{
		return $this->m_print_role_form;
	}

	function get_print_user_form()
	{
		return $this->m_print_user_form;
	}

	function set_override_plugin_deactivation($b)
	{
		$this->m_override_plugin_deactivation = $b;
	}
}

?>